/* -*- Mode: c++ -*- */
/*
 * Copyright 2001 Free Software Foundation, Inc.
 * 
 * This file is part of GNU Radio
 * 
 * GNU Radio is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * GNU Radio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with GNU Radio; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <GrAudioSource.h>
#include <GrAudioSink.h>
#include <GrSimpleScopeSink.h>
#include <GrFFTSink.h>
#include <VrConnect.h>
#include <VrMultiTask.h>
#include <VrGUI.h> 

#include <GrHilbert.h>
#include <GrSSBMod.h>


#define SAMPLING_FREQUENCY      16000
#define	AMPLITUDE		 4000

#define FREQ_OFFSET             400

const char audiodev[] = "/dev/dsp";

int main(int argc, char **argv) {
  VrGUI *guimain = new VrGUI(argc, argv);
  VrGUILayout *horiz = guimain->top->horizontal();
  VrGUILayout *vert = horiz->vertical();
  VrGUILayout *vert2 = horiz->horizontal()->vertical();

  VrSource<short> *source = new GrAudioSource<short>(SAMPLING_FREQUENCY,
						     1, /* num streams */
						     32767, /* output range */
						     audiodev);

  GrHilbert<short> *hilb = new GrHilbert<short>(31);
  GrSSBMod<short> *shifter = new GrSSBMod<short>(2*M_PI*FREQ_OFFSET/SAMPLING_FREQUENCY,1);

  VrSink<short> *scope_sink = new GrSimpleScopeSink<short>(vert, -AMPLITUDE, AMPLITUDE, 500);
  VrSink<short> *scope_sink2 = new GrSimpleScopeSink<short>(vert, -AMPLITUDE, AMPLITUDE, 500);

  VrSink<short> *fft_sink = new GrFFTSink<short>(vert2, 20, 100, 512);
  VrSink<short> *fft_sink2 = new GrFFTSink<short>(vert2, 20, 100, 512);
  VrSink<VrComplex> *fft_sink3 = new GrFFTSink<VrComplex>(vert2, 20, 100, 512);

  VrSink<short> *audio_sink = new GrAudioSink<short>(32767, /* input range */
						     audiodev);

  NWO_CONNECT (source, scope_sink);
  NWO_CONNECT (source, fft_sink);
  NWO_CONNECT (source, hilb);
  NWO_CONNECT (hilb,shifter);
  NWO_CONNECT (hilb,fft_sink3);
  NWO_CONNECT (shifter,audio_sink);
  NWO_CONNECT (shifter,fft_sink2);
  NWO_CONNECT (shifter,scope_sink2);

  VrMultiTask *m = new VrMultiTask ();

  m->add (scope_sink);
  m->add (scope_sink2);

  m->add (fft_sink);
  m->add (fft_sink2);
  m->add (fft_sink3);

  m->add (audio_sink);

  m->start();

  guimain->start();

  while (1) {
    guimain->processEvents(10 /*ms*/);
    m->process();
  }
}
