/* -*- Mode: c++ -*- */
/*
 * Copyright 2001 Free Software Foundation, Inc.
 * 
 * This file is part of GNU Radio
 * 
 * GNU Radio is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * GNU Radio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with GNU Radio; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
/*
 *  Copyright 1997 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and its
 *  documentation for any purpose is hereby granted without fee, provided that
 *  the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */


#ifndef _GRNOISESOURCE_H_
#define _GRNOISESOURCE_H_

#include <VrSource.h>
#include <gr_random.h>

enum { GR_GAUSSIAN, GR_LAPLACIAN, GR_IMPULSE };

template<class oType> 
class GrNoiseSource : public VrSource<oType> {
protected:
  int type;
  float amp;
  gr_random RNG;
public:
  virtual const char *name() { return "GrNoiseSource"; }
  virtual int work2(VrSampleRange output, void *o[]);
  
  GrNoiseSource(double sampfreq, int type, float amp, int seed = 3021)  
    :type(type),amp(amp) {
    setSamplingFrequency(sampfreq);
    RNG.reseed(seed);
  }
};

// Specialization for Complex Output
/*
  template<>
class GrNoiseSource<VrComplex> : public VrSource<VrComplex> {
protected:
  int type;
  float amp;
  gr_random RNG;
public:
  virtual const char *name() { return "GrNoiseSource"; }
  virtual int work2(VrSampleRange output, void *o[]);

  GrNoiseSource(double sampfreq, int type, float amp, int seed = 3021)
    :type(type),amp(amp) {
    setSamplingFrequency(sampfreq);
    RNG.reseed(seed);
  }
};
*/

template<class oType> int
GrNoiseSource<oType>::work2(VrSampleRange output, void *ao[])
{ 
  oType **o = (oType **)ao;
  unsigned int size = output.size;
  switch(type)
    {
    case GR_GAUSSIAN:
      for (unsigned int i=0; i<size; i++) 
	*o[0]++ = (oType)(amp * RNG.gasdev());
      break;
    case GR_LAPLACIAN:
      for (unsigned int i=0; i<size; i++) 
	*o[0]++ = (oType)(amp * RNG.laplacian());
      break;
    case GR_IMPULSE:    // FIXME changeable impulse settings
      for (unsigned int i=0; i<size; i++) 
	*o[0]++ = (oType)(amp * RNG.impulse(9));
      break;
    default:
      cerr << "Unsupported Noise Type: " << type << endl;
      exit(-1);
    }
  return output.size;              // Others should use the slower mod method
}

template<> int
GrNoiseSource<VrComplex>::work2(VrSampleRange output, void *ao[])
{
  VrComplex **o = (VrComplex **)ao;
  unsigned int size = output.size;
  assert(type == GR_GAUSSIAN);
  for(unsigned int i=0; i<size; i++)
    *o[0]++ = (amp * RNG.rayleigh_complex());
  return size;
}

#endif 
