#
# Copyright 2003 Free Software Foundation, Inc.
# 
# This file is part of GNU Radio
# 
# GNU Radio is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# GNU Radio is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with GNU Radio; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
# 


def num_to_str (n):
    '''Convert a number to a string in engineering notation.  E.g., 5e-9 -> 5n'''
    m = abs(n)
    if m >= 1e9:
        return "%gG" % (n * 1e-9)
    elif m >= 1e6:
        return "%gM" % (n * 1e-6)
    elif m >= 1e3:
        return "%gk" % (n * 1e-3)
    elif m >= 1:
        return "%g" % (n)
    elif m >= 1e-3:
        return "%gm" % (n * 1e3)
    elif m >= 1e-6:
        return "%gu" % (n * 1e6)        # where's that mu when you need it (unicode?)
    elif m >= 1e-9:
        return "%gn" % (n * 1e9)
    else:
        return "%g" % (n)


def str_to_num (s):
    '''Convert a string in engineering notation to a number.  E.g., '15ms' -> 15e-3'''
    return float (s)

